using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using Cursed.view;
using Cursed.gob;
using Cursed.gob.mob;
using Cursed.input;
using Cursed.gob.tile;
using Cursed.gob.mob.kob;
using Cursed.sound;
using Cursed.editor;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using Cursed.input.ai; 
using Cursed.view.renders;
using System.Windows;
using Cursed.col;
using Cursed.geom;
using System.Xml;
using Cursed.ui;


namespace Cursed
{

    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class CursedGame : Microsoft.Xna.Framework.Game
    {

        Profiler profiler;
        bool f1Down;

        GraphicsDeviceManager graphics;

        #region gamevars

        SpriteBatch spriteBatch;

        OrthoCamera camera;
        //CameraDetector cameraDetector;

        HUD Hud;

        readonly object gobLock = new object();

        Dictionary<Render.RenderType, Render> renders;

        ICollection<Gob> gobsToAdd;
        ICollection<Gob> gobsToRemove;
        ICollection<Gob> gobs;

        ICollection<Mob> mobs;
        ICollection<Tile> tiles;
        Dictionary<string, List<Gob>> labeledGobs;

        ICollection<GobInfo> incomingLoadGobInfos;

        Druid druidPlayer;
        string spawnLabel;

        ICollection<Controller> controllersToAdd;
        ICollection<Controller> controllersToRemove;
        ICollection<Controller> controllers;

        ICollider collider;

        Cursed.sound.SoundBank soundBank;
        SoundManager soundManager;

        string music;

        SaveUtil saveUtil;
        Cursed.view.Console console;
        Editor editor = null;

        #region temporary
        public static Color FogColor = new Color(.2f, .22f, .2f);
        #endregion

        Effect effect;

        public static TextureLib textureLib;

        bool paused;
        Random random;

        public static CursedGame TheGame;
        #endregion

        public CursedGame()
            : base()
        {
            TheGame = this;

            profiler = new Profiler(true);

            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            Components.Add(new GamerServicesComponent(this));
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            graphics.PreferredBackBufferWidth = 800;
            graphics.PreferredBackBufferHeight = 600;

            //this.IsFixedTimeStep = true;
            //this.TargetElapsedTime = TimeSpan.FromMilliseconds(1000.0f / 65.0f);
            //graphics.SynchronizeWithVerticalRetrace = true;

            if (Const.FullScreen)
            {
                foreach (DisplayMode displayMode in GraphicsAdapter.DefaultAdapter.SupportedDisplayModes)
                {
                    if (displayMode.Width == 1240 && displayMode.Height == 1024)
                    {
                        graphics.PreferredBackBufferWidth = displayMode.Width;
                        graphics.PreferredBackBufferHeight = displayMode.Height;
                    }
                }
                graphics.ToggleFullScreen();
                graphics.IsFullScreen = true;
            }
            graphics.ApplyChanges();
            graphics.GraphicsDevice.RenderState.CullMode = CullMode.None;
            Window.Title = "Cursed!";

            renders = new Dictionary<Render.RenderType, Render>();
            gobs = new HashSet<Gob>();
            mobs = new HashSet<Mob>();
            tiles = new HashSet<Tile>();
            labeledGobs = new Dictionary<string, List<Gob>>();
            controllers = new HashSet<Controller>();

            gobsToAdd = new HashSet<Gob>();
            gobsToRemove = new HashSet<Gob>();
            controllersToAdd = new HashSet<Controller>();
            controllersToRemove = new HashSet<Controller>();

            incomingLoadGobInfos = null;

            paused = false;
            random = new Random();

            druidPlayer = null;
            spawnLabel = "";

            saveUtil = new SaveUtil(this);
            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            MediaPlayer.IsRepeating = true;

            camera = new OrthoCamera(Const.ViewSize, Window);

            effect = Content.Load<Effect>("effects/BasicEffect");

            textureLib = new TextureLib(Content);
            
            SpriteRender sr = new SpriteRender(GraphicsDevice, effect);
            renders[Render.RenderType.SpriteRender] = sr;

            TrailRender tr = new TrailRender(effect, GraphicsDevice);
            renders[Render.RenderType.TrailRender] = tr;

            DialogRender dr = new DialogRender(this);
            renders[Render.RenderType.DialogRender] = dr;			

            //collider = new QTCollider(this);
            collider = new Box2DCollider(this);
		            
            Hud = new HUD();
            incomingLoadGobInfos = Levels.StartScreen();

            soundBank = new Cursed.sound.SoundBank(Content);
            soundManager = new SoundManager();

            console = new Cursed.view.Console(this);

            
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            if (!Const.UpdateWithDraw)
                doUpdate(gameTime);
        }
        protected void doUpdate(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            profiler.Tick();
            long bd = profiler.Stop("BetweenUpdate");
            //System.Console.WriteLine("between updates " + bd);
            profiler.Start("Update");

            checkLoadGobs();

            if (!f1Down && Keyboard.GetState()[Keys.F1] == KeyState.Down)
            {
                profiler.Print();
            }
            f1Down = Keyboard.GetState()[Keys.F1] == KeyState.Down;
            if (Keyboard.GetState()[Keys.F2] == KeyState.Down)
            {
                profiler.Reset();
            }

            updateGobList();

            if (Paused)
                gameTime = new GameTime(TimeSpan.Zero, gameTime.ElapsedRealTime, TimeSpan.Zero, TimeSpan.Zero);

            foreach (Render render in renders.Values)
            {
                render.Reposition(camera);
            }

            profiler.Start("controllers");
            foreach (Controller controller in controllers)
            {
                controller.act(gameTime);
            }
            profiler.Stop("controllers");

            profiler.Start("collider.handleCollisions");
            collider.Update(gameTime);
            profiler.Stop("collider.handleCollisions");

            Bounds active = ActiveRegion;

            foreach (Gob gob in gobs)
            {
                if (gob.Inert || !active.Intersects(gob.Bounds))
                    continue;
                profiler.Start("gob.Update");
                gob.PreUpdate();
                gob.Update(gameTime);
                gob.PostUpdate();
                profiler.Stop("gob.Update");
            }

            camera.trackFocus();

            base.Update(gameTime);
            long dur = profiler.Stop("Update");
            //System.Console.WriteLine("update " + dur);
            profiler.Start("BetweenUpdate");
        }

        #region draw
        class GobDepthComparer : IComparer<Gob>
        {
            public int Compare(Gob g1, Gob g2)
            {
                if (g1.GobInfo.Depth < g2.GobInfo.Depth)
                    return 1;
                if (g1.GobInfo.Depth > g2.GobInfo.Depth)
                    return -1;
                return g1.GetHashCode() - g2.GetHashCode();
            }
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected  override void Draw(GameTime gameTime)
        {
            if (Const.UpdateWithDraw)
                doUpdate(gameTime);
            long bd = profiler.Stop("BetweenDraw");
            //System.Console.WriteLine("between draws " + bd);
            profiler.Start("Draw");
            GraphicsDevice.Clear(Color.Black);
            GraphicsDevice.RenderState.AlphaBlendEnable = true;
            GraphicsDevice.RenderState.SourceBlend = Blend.SourceAlpha;
            GraphicsDevice.RenderState.DestinationBlend = Blend.InverseSourceAlpha;
            GraphicsDevice.RenderState.DepthBufferEnable = false;
            
            SortedList<Gob, Gob> sortedGobs = new SortedList<Gob, Gob>(new GobDepthComparer());

            Bounds bounds = camera.Bounds;
            ICollection<Gob> gobsInView = collider.GobsInBounds(bounds);
            Profiler.Start("glist.Sort");
            foreach (Gob gob in gobsInView)
                sortedGobs.Add(gob, gob);

            Profiler.Stop("glist.Sort");


            Profiler.Start("gobs Draw");
            foreach (KeyValuePair<Gob,Gob> gobPair in sortedGobs)
            {
                gobPair.Value.Draw(gameTime);
            }
            Profiler.Stop("gobs Draw");

            Profiler.Start("draw other");
            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Texture, SaveStateMode.SaveState);
            spriteBatch.Draw(textureLib.GetTexture("fog"),
                new Rectangle(
                    0,
                    0,
                    GraphicsDevice.Viewport.Width,
                    GraphicsDevice.Viewport.Height),
                    FogColor);
            spriteBatch.End();
            if (FogColor.R > 51.2)
            {
                FogColor = new Color((byte)(FogColor.R - 5), FogColor.G, FogColor.B);
            }
            Hud.Draw(spriteBatch, gameTime);

            console.Draw(spriteBatch);
            editor.Draw(spriteBatch);

            Profiler.Stop("draw other");
            
            long dur = profiler.Stop("Draw");

            //System.Console.WriteLine("draw " + dur);
            
            base.Draw(gameTime);
            profiler.Start("BetweenDraw");
        }
        #endregion

        #region insert

        public void LoadLevel(string identifier)
        {
            if (identifier == "new")
            {
                incomingLoadGobInfos = new List<GobInfo>();
                return;
            }
            if (identifier == "Test1")
            {
                incomingLoadGobInfos = Levels.InfoTest1();
                return;
            }
            if (identifier == "StartScreen")
            {
                incomingLoadGobInfos = Levels.StartScreen();
                return;
            }

            XmlDocument saveDoc = new XmlDocument();
            saveDoc.Load("../../../../levels/" + identifier + ".xml");
            XmlNode infoNodeList = saveDoc.FirstChild;

            incomingLoadGobInfos = new List<GobInfo>();
            foreach (XmlNode infoNode in infoNodeList)
            {
                GobInfo info = GobInfo.CreateGobInfoFromXML(infoNode);
                incomingLoadGobInfos.Add(info);
            }
        }

        public void SaveLevel(string identifier)
        {

            TextWriter outFile = new StreamWriter("../../../../levels/"+identifier + ".xml");
            outFile.Write("<CursedSave>\n");
            foreach (Gob gob in gobs)
            {
                if (gob.Transient)
                    continue;
                if (gob == DruidPlayer)
                    continue;
                GobInfo info = gob.GobInfo;
                string xml = info.XML;
                outFile.Write(xml);
            }
            outFile.Write("</CursedSave>\n");
            outFile.Close();
        }

        public ICollection<GobInfo> IncomingLoadGobInfos
        {
            set { incomingLoadGobInfos = value; }
        }

        void checkLoadGobs()
        {
            if (incomingLoadGobInfos != null)
            {
                clearAllObjects();
                Druid player = null;
                foreach (GobInfo info in incomingLoadGobInfos)
                {
                    Gob gob = info.Create(this);

                    if (gob is Druid)
                    {
                        Druid druid = (Druid)gob;
                        if (druid.DruidInfo.IsPlayer)
                            player = druid;
                    }

                    AddGob(gob);
                }
                incomingLoadGobInfos = null;
                updateGobList();

                if (player == null && DruidPlayer != null)
                {
                    AddGob(DruidPlayer);
                }
                else
                {
                    DruidPlayer = player;
                }
                if (DruidPlayer != null)
                {
                    Camera.PushFocus(DruidPlayer);
                    Hud.AddComponenet(new HealthBar(new Rectangle(0, 0, 200, 20), DruidPlayer));
                    Hud.AddComponenet(new ManaBar(new Rectangle(0, 20, 200, 20), DruidPlayer));
                }

                if (SpawnLabel != "")
                {
                    ICollection<Gob> spawnGobs = getLabeledGobs(SpawnLabel);
                    foreach (Gob gob in spawnGobs)
                    {
                        if (gob is Entrance)
                        {
                            DruidPlayer.GobInfo.Loc = gob.GobInfo.Loc;
                        }
                    }
                }

                editor = new Editor(this, Vector2.Zero);
            }
        }

        void clearAllObjects()
        {
            gobs.Clear();
            mobs.Clear();
            tiles.Clear();
            labeledGobs.Clear();
            gobsToAdd.Clear();
            gobsToRemove.Clear();

            controllersToAdd.Clear();
            controllersToRemove.Clear();
            controllers.Clear();

            collider.Clear();
        }
        void updateGobList()
        {
            Profiler.Start("updateGobList locking");
            lock (gobLock)
            {
                Profiler.Stop("updateGobList locking");
                foreach (Controller controller in controllersToAdd)
                {
                    controllers.Add(controller);
                }
                controllersToAdd.Clear();
                foreach (Controller controller in controllersToRemove)
                {
                    controllers.Remove(controller);
                }
                foreach (Gob gob in gobsToAdd)
                {
                    if (gob is Tile)
                    {
                        tiles.Add((Tile)gob);
                        collider.AddTile((Tile)gob);
                    }

                    if (gob is Mob)
                    {
                        mobs.Add((Mob)gob);
                        collider.AddMob((Mob)gob);
                    }
                    if (gob.GobInfo.Label != "")
                    {
                        if (!labeledGobs.ContainsKey(gob.GobInfo.Label))
                        {
                            labeledGobs[gob.GobInfo.Label] = new List<Gob>();
                        }
                        labeledGobs[gob.GobInfo.Label].Add(gob);
                    }

                    gobs.Add(gob);
                    gob.Update(new GameTime());
                }
                gobsToAdd.Clear();

                foreach (Gob gob in gobsToRemove)
                {
                    if (gob is Tile)
                    {
                        tiles.Remove((Tile)gob);
                        collider.RemoveTile((Tile)gob);
                    }

                    if (gob is Mob)
                    {
                        mobs.Remove((Mob)gob);
                        collider.RemoveMob((Mob)gob);
                    }

                    if (gob.GobInfo.Label != null)
                    {
                        if (labeledGobs.ContainsKey(gob.GobInfo.Label))
                        {
                            labeledGobs[gob.GobInfo.Label].Remove(gob);
                        }
                    }

                    gobs.Remove(gob);
                }
                gobsToRemove.Clear();
            }
        }

        public void AddController(Controller controller)
        {
            if (controller == null)
            {
                return;
            }
            controllersToAdd.Add(controller);

        }

        public void RemoveController(Controller controller)
        {
            if (controller == null)
                return;
            controllersToRemove.Remove(controller);
            controller.cleanup();
        }

        public void AddGob(Gob gob)
        {
            if (gob == null)
                return;
            gobsToAdd.Add(gob);
            gob.Load();
            gob.Update(new GameTime());
        }

        public void RemoveGob(Gob gob)
        {
            if (gob == null)
                return;
            if (gob == DruidPlayer)
                DruidPlayer = null;
            gobsToRemove.Add(gob);
            gob.Unload();
        }

        public ICollection<Gob> getLabeledGobs(string label)
        {
            if (!labeledGobs.ContainsKey(label))
            {
                labeledGobs[label] = new List<Gob>();
            }
            return labeledGobs[label];
        }

        public ICollection<Mob> MobsInBounds(Bounds bounds)
        {
            return collider.MobsInBounds(bounds);
        }

        public ICollection<Gob> GobsInBounds(Bounds bounds)
        {
            return collider.GobsInBounds(bounds);
        }

        public ICollection<Tile> TilesAt(int x, int y)
        {
            return collider.TilesAt(x, y);
        }

        public void CollectMobsInBounds(Bounds bounds, ICollection<Mob> pool)
        {
            collider.CollectMobsInBounds(bounds, pool);
        }

        public void CollectGobsInBounds(Bounds bounds, ICollection<Gob> pool)
        {
            collider.CollectGobsInBounds(bounds, pool);
        }

        #endregion

        #region accessors

        public string Music
        {
            get { return music; }
            set
            {
                if (music == value)
                    return;
                music = value;
                MediaPlayer.Stop();
                MediaPlayer.Play(Content.Load<Song>("music\\" + music));
            }
        }

        public Bounds ActiveRegion
        {
            get
            {
                return Bounds.Everything;

                //float w = Camera.ViewWidth + 2 * Const.ActiveRegionBuffer;
                //float h = Camera.ViewHeight + 2 * Const.ActiveRegionBuffer;
                //return new Bounds(Camera.Position, w, h);
            }
        }

        public Render getRender(Render.RenderType type)
        {
            return renders[type];
        }

        public bool Paused
        {
            get { return paused; }
            set { paused = value; }
        }

        public Profiler Profiler
        {
            get { return profiler; }
        }
        public Cursed.sound.SoundBank SoundBank
        {
            get { return soundBank; }
        }
        public SoundManager SoundManager
        {
            get { return soundManager; }
        }
        public ICamera Camera
        {
            get { return camera; }
        }

        public Random Random
        {
            get { return random; }
        }

        public Vector2 RandomVector2
        {
            get
            {
                Vector2 v = new Vector2(0, 1);
                float theta = (float)(Random.NextDouble() * MathHelper.Pi * 2);
                return Vector2.Transform(v, Matrix.CreateRotationZ(theta));
            }
        }

        public ICollection<Gob> Gobs
        {
            get { return gobs; }
        }

        public ICollection<Mob> Mobs
        {
            get { return mobs; }
        }

        public Druid DruidPlayer
        {
            get { return druidPlayer; }
            set { 
                druidPlayer = value;
                camera.PushFocus(druidPlayer);
            }
        }

        public string SpawnLabel
        {
            get { return spawnLabel; }
            set { spawnLabel = value; }
        }

        public ICollection<Tile> Tiles
        {
            get { return tiles; }
        }
        public SpriteBatch SpriteBatch
        {
            get { return spriteBatch; }
        }
        #endregion
    }
}
