﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Cursed
{
    public class Profiler
    {
        bool enabled;

        Dictionary<string, long> startTimes;
        Dictionary<string, long> totalTimes;
        Dictionary<string, long> maxTimes;
        Dictionary<string, long> counts;
        long ticks;

        public Profiler(bool enabled)
        {
            this.enabled = enabled;
            Reset();
        }

        public void Reset()
        {
            startTimes = new Dictionary<string, long>();
            totalTimes = new Dictionary<string, long>();
            maxTimes = new Dictionary<string, long>();
            counts = new Dictionary<string, long>();
            ticks = 0;
        }

        public void Tick()
        {
            ticks++;
        }

        public void Start(string label)
        {
            if (!enabled) return;
            startTimes[label] = DateTime.Now.Millisecond;
        }
        public long Stop(string label)
        {
            if (!enabled) return 0;
            if (!startTimes.ContainsKey(label))
                return 0;
            long duration = DateTime.Now.Millisecond - startTimes[label];
            if (duration < 0)
                return 0;
            if (!totalTimes.ContainsKey(label))
            {
                totalTimes[label] = 0;
                maxTimes[label] = 0;
                counts[label] = 0;
            }
            totalTimes[label] += duration;
            if (duration > maxTimes[label])
                maxTimes[label] = duration;
            counts[label]++;
            return duration;
        }
        public void Print()
        {
            if (!enabled)
                System.Console.WriteLine("Profiler not enabled");
            System.Console.WriteLine("Profiler:");
            foreach (string label in totalTimes.Keys)
            {
                long totalDuration = totalTimes[label];
                long count = counts[label];
                float time = 1.0f * totalDuration / count;
                float perTick = 1.0f * totalDuration / ticks;
                System.Console.WriteLine(" " + label + ": " + perTick + "ms/tick, "+totalDuration * .001f + "s/" + count + ", " + time + "ms avg, " + maxTimes[label] + "ms max");
            }
        }

        public Boolean Enabled
        {
            get { return enabled; }
            set { enabled = value; }
        }
    }
}
