﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.gob.mob;
using Microsoft.Xna.Framework;
using Cursed.gob;
using Cursed.gob.tile;
using Cursed.geom;

namespace Cursed.col
{
    class QTNode
    {
        Bounds bounds;
        QTNode[] childTrees = {null, null, null, null};
        HashSet<Mob> gobs;
        bool hasChildren;
        float maxSize;


        public QTNode()
        {
            gobs = new HashSet<Mob>();
            maxSize = 0;
            hasChildren = false;
            //System.Console.WriteLine("new qtnode "+(++count));
        }

        public void Initialize(Bounds bounds)
        {
            gobs.Clear();
            hasChildren = false;
            maxSize = 0;
            this.bounds = bounds;
        }

        public void Clear()
        {
            gobs.Clear();
            for (int i=0; i<4; i++)
            {
                if (childTrees[i] != null)
                    childTrees[i].Clear();
            }
        }

        public void Add(Mob gob)
        {
            if (!hasChildren)
            {

                maxSize = Math.Max(maxSize, gob.Bounds.Width);
                maxSize = Math.Max(maxSize, gob.Bounds.Height);

                if (gobs.Count == Const.MaxQTChildren && (bounds.Width > maxSize && bounds.Height > maxSize))
                {
                    hasChildren = true;
                    float cx = bounds.Center.X;
                    float cy = bounds.Center.Y;
                    CursedGame.TheGame.Profiler.Start("qt alloc");
                    //subtrees = new QTNode[4];
                    if (childTrees[0] == null)
                    {
                        childTrees[0] = new QTNode();
                        childTrees[1] = new QTNode();
                        childTrees[2] = new QTNode();
                        childTrees[3] = new QTNode();
                    }
                    childTrees[0].Initialize(new Bounds(bounds.MinX, bounds.MinY, cx, cy));
                    childTrees[1].Initialize(new Bounds(bounds.MinX, cy, cx, bounds.MaxY));
                    childTrees[2].Initialize(new Bounds(cx, bounds.MinY, bounds.MaxX, cy));
                    childTrees[3].Initialize(new Bounds(cx, cy, bounds.MaxX, bounds.MaxY));
                    
                    CursedGame.TheGame.Profiler.Stop("qt alloc");

                    foreach (Mob g in gobs)
                        addSub(g);
                    addSub(gob);
                }
                else
                {
                    gobs.Add(gob);
                }
            }
            else
            {
                addSub(gob);
            }
        }

        void addSub(Mob gob)
        {
            Bounds bounds = gob.Bounds;
            foreach (QTNode qtNode in childTrees)
            {
                if (qtNode.bounds.Intersects(bounds))
                    qtNode.Add(gob);
            }
        }

        public void Collect(ICollection<Mob> pool, Bounds bounds)
        {
            if (!hasChildren)
            {
                foreach (Mob gob in gobs)
                {
                    if (bounds.Intersects(gob.Bounds))
                        pool.Add(gob);
                }
            }
            else
            {
                foreach (QTNode qtNode in childTrees)
                {
                    if (qtNode.bounds.Intersects(bounds))
                        qtNode.Collect(pool, bounds);
                }
            }
        }

        public void Collect(ICollection<Gob> pool, Bounds bounds)
        {
            if (!hasChildren)
            {
                foreach (Mob gob in gobs)
                {
                    if (bounds.Intersects(gob.Bounds))
                        pool.Add(gob);
                }
            }
            else
            {
                foreach (QTNode qtNode in childTrees)
                {
                    if (qtNode.bounds.Intersects(bounds))
                        qtNode.Collect(pool, bounds);
                }
            }
        }
    }

    public class QTCollider : ICollider
    {
        protected ICollection<Mob> mobs;
        QTNode root;

        ICollection<Tile> empty;
        protected ICollection<Tile> tiles;
        ICollection<Tile>[,] tileGrid;
        int minTileX;
        int maxTileX;
        int spanTileX;
        int minTileY;
        int maxTileY;
        int spanTileY;

        CursedGame game;

        public QTCollider(CursedGame game)
        {
            this.game = game;
            empty = new List<Tile>();
            Clear();
        }

        public virtual void Clear()
        {
            mobs = new HashSet<Mob>();
            root = null;
            tiles = new HashSet<Tile>();
            minTileX = int.MaxValue;
            maxTileX = -1 * int.MaxValue;
            minTileY = int.MaxValue;
            maxTileY = -1 * int.MaxValue;
        }

        void repackTiles()
        {
            minTileX = int.MaxValue;
            maxTileX = -1 * int.MaxValue;
            minTileY = int.MaxValue;
            maxTileY = -1 * int.MaxValue;
            foreach (Tile tile in tiles)
            {
                if (tile.X < minTileX) minTileX = tile.X;
                if (tile.Y < minTileY) minTileY = tile.Y;
                if (tile.X > maxTileX) maxTileX = tile.X;
                if (tile.Y > maxTileY) maxTileY = tile.Y;
            }
            spanTileX = maxTileX - minTileX + 1;
            spanTileY = maxTileY - minTileY + 1;

            if (spanTileX > 0)
            {
                tileGrid = new ICollection<Tile>[spanTileX, spanTileY];
                for (int x = 0; x < spanTileX; x++)
                    for (int y = 0; y < spanTileY; y++)
                    {
                        tileGrid[x, y] = new HashSet<Tile>();
                    }
                foreach (Tile tile in tiles)
                {
                    tileGrid[tile.X - minTileX, tile.Y - minTileY].Add(tile);
                }
            }
        }
        public ICollection<Tile> TilesAt(int x, int y)
        {
            if (x < minTileX || y < minTileY || x > maxTileX || y > maxTileY)
                return empty;
            return tileGrid[x - minTileX, y - minTileY];
        }

        #region ICollider Members

        protected void buildQT()
        {
            Bounds activeRegion = game.ActiveRegion;

            IEnumerator<Mob> mobEnumerator = mobs.GetEnumerator();
            if (!mobEnumerator.MoveNext())
                return;

            game.Profiler.Start("Bounds union");
            Bounds bounds = Bounds.Nothing;
            foreach (Mob gob in mobs)
            {
                if (!activeRegion.Intersects(gob.Bounds))
                    continue;
                bounds = Bounds.Union(bounds, gob.Bounds);
            }
            game.Profiler.Stop("Bounds union");

            game.Profiler.Start("qt build");
            if (root == null)
                root = new QTNode();
            root.Clear();
            root.Initialize(bounds);
            foreach (Mob gob in mobs)
            {
                if (!activeRegion.Intersects(gob.Bounds))
                    continue;
                root.Add(gob);
            }
            game.Profiler.Stop("qt build");
        }

        public virtual void HandleCollisions()
        {

            buildQT();

            Bounds activeRegion = game.ActiveRegion;

            game.Profiler.Start("mob touch");
            foreach (Mob mob in mobs)
            {
                if (mob.IsEffect || !activeRegion.Intersects(mob.Bounds))
                    continue;

                mob.clearTouch();

                HashSet<Gob> boundTouchedGobs = new HashSet<Gob>();
                root.Collect(boundTouchedGobs, mob.Bounds);
                foreach (Gob o in boundTouchedGobs)
                {
                    if (o.IsEffect || o == mob)
                        continue;
                    if (mob.touches(o))
                        mob.Touch(o);
                }
            }
            game.Profiler.Stop("mob touch");

            foreach (Tile tile in tiles)
                tile.clearTouch();

            game.Profiler.Start("tile touch");
            foreach (Mob mob in mobs)
            {
                if (!activeRegion.Intersects(mob.Bounds))
                    continue;

                int minX = (int)Math.Ceiling(mob.Bounds.MinX - .5);
                int minY = (int)Math.Ceiling(mob.Bounds.MinY - .5);
                int maxX = (int)Math.Floor(mob.Bounds.MaxX + .5);
                int maxY = (int)Math.Floor(mob.Bounds.MaxY + .5);

                if (minX < minTileX)
                    minX = minTileX;
                if (minY < minTileY)
                    minY = minTileY;
                if (maxX > maxTileX)
                    maxX = maxTileX;
                if (maxY > maxTileY)
                    maxY = maxTileY;

                Rect r = new Rect(Vector2.Zero, 1, 1);
                for (int x = minX; x <= maxX; x++)
                {
                    for (int y = minY; y <= maxY; y++)
                    {
                        r.setTransform(new Vector2(x, y), 0, 1);
                        if (mob.Shape.intersects(r))
                        {
                            ICollection<Tile> tilesHere = TilesAt(x, y);
                            foreach (Tile tile in tilesHere)
                            {
                                mob.Touch(tile);
                                tile.Touch(mob);
                            }
                        }
                    }
                }
            }

            for (int i = 0; i < Const.PushIterations; i++)
            {
                foreach (Mob mob in mobs)
                {
                    mob.Push(Const.PushFactor);
                }
            }
            game.Profiler.Stop("tile touch");
        }
        #region insert
        public virtual void AddMob(Cursed.gob.mob.Mob mob)
        {
            mobs.Add(mob);
        }

        public virtual void RemoveMob(Cursed.gob.mob.Mob mob)
        {
            mobs.Remove(mob);
        }

        public virtual void AddTile(Cursed.gob.tile.Tile tile)
        {
            tiles.Add(tile);
            if (tile.TileInfo.X > maxTileX || tile.TileInfo.Y > maxTileY ||
                tile.TileInfo.X < minTileX || tile.TileInfo.Y < minTileY)
            {
                repackTiles();
            }
            else
            {
                tileGrid[tile.X - minTileX, tile.Y - minTileY].Add(tile);
            }
        }

        public virtual void RemoveTile(Cursed.gob.tile.Tile tile)
        {
            tiles.Remove(tile);
            TilesAt(tile.X, tile.Y).Remove(tile);
        }
        #endregion

        public ICollection<Mob> MobsInBounds(Bounds bounds)
        {
            HashSet<Mob> pool = new HashSet<Mob>();
            CollectMobsInBounds(bounds, pool);
            return pool;
        }
        public void CollectMobsInBounds(Bounds bounds, ICollection<Mob> pool)
        {
            if (root == null) return;
            game.Profiler.Start("root.Collect");
            root.Collect(pool, bounds);
            game.Profiler.Stop("root.Collect");
        }
        public ICollection<Gob> GobsInBounds(Bounds bounds)
        {
            HashSet<Gob> pool = new HashSet<Gob>();
            CollectGobsInBounds(bounds, pool);
            return pool;
        }
        public void CollectGobsInBounds(Bounds bounds, ICollection<Gob> pool)
        {
            if (root == null) return;
            game.Profiler.Start("root.Collect");
            root.Collect(pool, bounds);
            game.Profiler.Stop("root.Collect");

            game.Profiler.Start("Tile collect");

            int minX = (int)Math.Ceiling(bounds.MinX - .5);
            int minY = (int)Math.Ceiling(bounds.MinY - .5);
            int maxX = (int)Math.Floor(bounds.MaxX + .5);
            int maxY = (int)Math.Floor(bounds.MaxY + .5);

            if (minX < minTileX)
                minX = minTileX;
            if (minY < minTileY)
                minY = minTileY;
            if (maxX > maxTileX)
                maxX = maxTileX;
            if (maxY > maxTileY)
                maxY = maxTileY;

            for (int x = minX; x <= maxX; x++)
            {
                for (int y = minY; y <= maxY; y++)
                {
                    ICollection<Tile> tilesHere = TilesAt(x, y);
                    foreach (Tile tile in tilesHere)
                        pool.Add(tile);
                }
            }


            game.Profiler.Stop("Tile collect");
        }

        #endregion

        #region ICollider Members


        public virtual void Update(GameTime gameTime)
        {
            float seconds = gameTime.ElapsedGameTime.Milliseconds * .001f;
            HandleCollisions();
            foreach (Mob mob in mobs)
            {
                mob.UpdatePosition(seconds);
            }
        }

        #endregion
    }
}
