﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.geom;
using Microsoft.Xna.Framework;

namespace Cursed.gob.mob
{
    public class TriggerZoneInfo : MobInfo
    {
        float width;
        float height;

        public TriggerZoneInfo()
            : base()
        {
            width = 1;
            height = 1;
            Label = "door";
        }

        public override Gob Create(CursedGame game)
        {
            return new TriggerZone(game, this);
        }

        public float Width
        {
            get { return width; }
            set { width = value; }
        }

        public float Height
        {
            get { return height; }
            set { height = value; }
        }
    }

    public class TriggerZone : Mob
    {
        public TriggerZone(CursedGame game, TriggerZoneInfo info)
            : base(game, info)
        {
            Transcendent = true;
        }

        public override void Load()
        {
            shape = new Rect(Vector2.Zero, TriggerZoneInfo.Width, TriggerZoneInfo.Height);
        }

        protected virtual bool canTrigger(Gob gob)
        {
            return !gob.Transcendent;
        }

        public override void Update(Microsoft.Xna.Framework.GameTime gameTime)
        {
            foreach (Gob gob in TouchedGobs)
            {
                if (canTrigger(gob))
                {
                    
                    GobInfo.Triggered = true;
                    break;
                }
            }

            if (GobInfo.Triggered)
            {
                ICollection<Gob> labelGobs = CursedGame.getLabeledGobs(GobInfo.Label);
                foreach (Gob gob in labelGobs)
                {
                    gob.GobInfo.Triggered = true;
                }
            }

            base.Update(gameTime);
        }

        public override void PostUpdate()
        {
            //not setting triggered to false
        }

        public TriggerZoneInfo TriggerZoneInfo
        {
            get { return (TriggerZoneInfo)info; }
        }
    }
}
