﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.geom;
using Microsoft.Xna.Framework;
using Cursed.view.renders;
using Cursed.input;
using Cursed.gob.mob.effects;
using Microsoft.Xna.Framework.Graphics;

namespace Cursed.gob.mob
{
    public class WandInfo : MobInfo
    {
        public WandInfo()
            : base()
        {
            Depth = Const.WandDepth;
        }

        public override Gob Create(CursedGame game)
        {
            return new Wand(game, this);
        }
    }

    public class Wand : Mob
    {
        MouseType mouseType;
        bool strong;

        public Wand(CursedGame game, WandInfo info)
            : base(game, info)
        {
            Transient = true;
            Transcendent = true;
            mouseType = MouseType.None;
            strong = false;
        }

        public override void Load()
        {
            shape = new Circle(Vector2.Zero, .05f);    
        }

        public override void Update(GameTime gameTime)
        {
            if (gameTime.ElapsedGameTime.Milliseconds > 0)
            {
                SparkInfo sparkInfo = new SparkInfo();
                sparkInfo.Loc = GobInfo.Loc;
                sparkInfo.Vel = CursedGame.RandomVector2;
                sparkInfo.Omega = 25 * (float)(CursedGame.Random.NextDouble() - .5);
                if (mouseType == MouseType.Offensive)
                    sparkInfo.Color = Color.Blue;
                if (mouseType == MouseType.Defensive)
                    sparkInfo.Color = Color.Black;
                if (mouseType == MouseType.None)
                    sparkInfo.Color = Color.White;
                if (strong)
                    sparkInfo.Scale *= 1.5f;
                sparkInfo.Transparency = 1f;
                Spark spark = (Spark)sparkInfo.Create(CursedGame);

                CursedGame.AddGob(spark);
            }
            base.Update(gameTime);
        }

        public WandInfo WandInfo
        {
            get { return (WandInfo)info; }
        }

        public MouseType MouseType
        {
            get { return mouseType; }
            set { mouseType = value; }
        }

        public bool Strong
        {
            get { return strong; }
            set { strong = value; }
        }
    }
}
