﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Cursed.geom;
using Cursed.view.renders;
using Microsoft.Xna.Framework.Graphics;

namespace Cursed.gob.mob
{
    public class WoodInfo : MobInfo
    {
        float decayRate = .4f;
        float hinderance = .5f;

        public WoodInfo()
            : base()
        {
            Stuck = true;
            Depth = Const.WoodDepth;
        }

        public override Gob Create(CursedGame game)
        {
            return new Wood(game, this);
        }

        public float DecayRate
        {
            get { return decayRate; }
            set { decayRate = value; }
        }

        public float Hinderance
        {
            get { return hinderance; }
            set { hinderance = value; }
        }
    }

    public class Wood : Mob
    {
        public Wood(CursedGame game, WoodInfo info)
            : base(game, info)
        {
            Transient = true;
            Transcendent = true;
        }

        public override void Load()
        {
            shape = new Circle(Vector2.Zero, .2f);
            renderInfo = new SpriteRenderInfo(shape.Bounds);
            renderInfo.Texture = CursedGame.textureLib.GetTexture("circle");
            renderInfo.Mask = new Color(92, 51, 23);
            addRender(CursedGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender), renderInfo);
        }

        public override void Update(GameTime gameTime)
        {
            foreach (Gob gob in TouchedGobs)
            {
                if (gob is Mob)
                {
                    Mob mob = (Mob)gob;
                    mob.ApplyDrag(WoodInfo.Hinderance * MobInfo.Scale);
                    //Vector2 f = mob.MobInfo.Vel * -1 * WoodInfo.Hinderance * MobInfo.Scale;
                    //mob.ApplyForce(f);
                }
            }

            MobInfo.Scale -= WoodInfo.DecayRate * gameTime.ElapsedGameTime.Milliseconds * .001f;
            if (MobInfo.Scale <= 0)
            {
                CursedGame.RemoveGob(this);
            }
            base.Update(gameTime);
        }

        public WoodInfo WoodInfo
        {
            get { return (WoodInfo)info; }
        }
    }
}
