﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using Cursed.view;
using Cursed.input;
using System.Runtime.Serialization;
using Cursed.geom;
using Cursed.view.renders;
using Cursed.gob.tile;
using Cursed.sound;
using Cursed.gob.mob.effects;
using Cursed.input.util;

namespace Cursed.gob.mob.kob
{
    public class DruidInfo : KobInfo
    {
        float healthRegen;
        float currentMana;
        float maxMana;
        float manaRegen;
        bool isPlayer;

        public DruidInfo()
            : base()
        {
            Thrust = 10;
            Drag = 3f;
            healthRegen = 0.05f;
            currentMana = 1;
            maxMana = 1;
            manaRegen = .05f;
            isPlayer = false;
            Depth = Const.DruidDepth;
        }

        public override Gob Create(CursedGame game)
        {
            return new Druid(game, this);
        }

        public float HealthRegen
        {
            get { return healthRegen; }
            set { healthRegen = value; }
        }
        public float CurrentMana
        {
            get { return currentMana; }
            set { currentMana = value; }
        }
        public float MaxMana
        {
            get { return maxMana; }
            set { maxMana = value; }
        }
        public float ManaRegen
        {
            get { return manaRegen; }
            set { manaRegen = value; }
        }
        public bool IsPlayer
        {
            get { return isPlayer; }
            set { isPlayer = value; }
        }
    }
    
    public class Druid : Kob
    {
        Wand wand;
        float timeToNextPoop;
        
        ICollection<Familiar> familiars;

        KeyDirectionController keyController;
        MouseController mouseController;
        MouseTrailController mouseTrailController;
        MouseGestureController gestureController;

        List<WGesture.Direction> familiarGesture;

        public Druid(CursedGame game, DruidInfo info)
            : base(game, info)
        {
            familiars = new HashSet<Familiar>();

            Transcendent = false;

            timeToNextPoop = 0;
        }
        public override void Load()
        {
            shape = new Circle(Vector2.Zero, .5f);
            //renderInfo = new SpriteRenderInfo(shape.Bounds);
            renderInfo = new SpriteRenderInfo(new Cursed.geom.Bounds(GobInfo.DrawCenter, GobInfo.DrawWidth, GobInfo.DrawHeight));
            renderInfo.Mask = Microsoft.Xna.Framework.Graphics.Color.Green;
            renderInfo.Texture = CursedGame.textureLib.GetTexture("circle");
            addRender(CursedGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender), renderInfo);

            WandInfo wandInfo = new WandInfo();
            wand = new Wand(CursedGame, wandInfo);
            CursedGame.AddGob(wand);

            hitSound = SoundIDs.DruidHit(CursedGame);

            if (DruidInfo.IsPlayer)
            {
                keyController = new KeyDirectionController();
                CursedGame.AddController(keyController);
                keyController.Push(this);

                mouseController = new MouseController(CursedGame.Camera, CursedGame.Window);
                CursedGame.AddController(mouseController);
                mouseController.Push(this);

                mouseTrailController = new MouseTrailController(CursedGame.Camera, CursedGame.Window);
                CursedGame.AddController(mouseTrailController);
                mouseTrailController.Push(this);

                gestureController = new MouseGestureController(CursedGame.Camera, CursedGame.Window);
                CursedGame.AddController(gestureController);
                gestureController.Push(this);
            }
            else
            {
                keyController = null;
                mouseController = null;
                mouseTrailController = null;
                gestureController = null;
            }

            familiarGesture = new List<WGesture.Direction>();
            familiarGesture.Add(WGesture.Direction.Up);
            familiarGesture.Add(WGesture.Direction.Right);

            SummonFamiliar(GobInfo.Loc + CursedGame.RandomVector2, 0);
        }
        public DruidInfo DruidInfo
        {
            get { return (DruidInfo)info; }
        }
        public override void Update(Microsoft.Xna.Framework.GameTime gameTime)
        {
            if (DruidInfo.CurrentHealth < DruidInfo.MaxHealth)
            {
                DruidInfo.CurrentHealth = Math.Min(DruidInfo.MaxHealth,
                    DruidInfo.CurrentHealth + DruidInfo.HealthRegen * gameTime.ElapsedGameTime.Milliseconds/1000f);
            }
            if (DruidInfo.CurrentMana < DruidInfo.MaxMana)
            {
                DruidInfo.CurrentMana = Math.Min(DruidInfo.MaxMana,
                    DruidInfo.CurrentMana + DruidInfo.ManaRegen * gameTime.ElapsedGameTime.Milliseconds / 1000f);
            }

            timeToNextPoop -= gameTime.ElapsedGameTime.Milliseconds * .001f;
            if (timeToNextPoop < 0)
            {
                LifeBlobInfo info = new LifeBlobInfo();
                Vector2 loc = GobInfo.Loc;
                loc.X += ((float)CursedGame.Random.NextDouble() - .5f) * .5f;
                loc.Y += ((float)CursedGame.Random.NextDouble() - .5f) * .5f;
                info.Loc = loc;
                info.Decay = Const.DruidPoopDecay;
                info.Amount = Const.DruidPoopAmount;
                LifeBlob blob = new LifeBlob(CursedGame, info);
                CursedGame.AddGob(blob);
                timeToNextPoop = Const.DruidRepoopTime;
            }

            base.Update(gameTime);
        }

        protected void castLife(Vector2 loc)
        {
            if (Ready && DruidInfo.CurrentMana >= Const.LifeSpellMana)
            {
                SoundIDs.LifeCast(CursedGame).play();
                LifeBlobInfo info = new LifeBlobInfo();
                info.Loc = loc;
                info.Amount = Const.LifeSpellAmount;
                LifeBlob blob = new LifeBlob(CursedGame, info);
                CursedGame.AddGob(blob);
                CoolDown += Const.LifeSpellCoolDown;
                DruidInfo.CurrentMana -= Const.LifeSpellMana;
            }
        }

        protected void castWood(Vector2 loc)
        {
            if (Ready && DruidInfo.CurrentMana >= Const.WoodSpellMana)
            {
                SoundIDs.WoodCast(CursedGame).play();
                WoodInfo info = new WoodInfo();
                info.Loc = loc;
                info.Theta = (float)CursedGame.Random.NextDouble() * MathHelper.Pi * 2;
                Wood wood = new Wood(CursedGame, info);
                CursedGame.AddGob(wood);
                CoolDown += Const.WoodSpellCoolDown;
                DruidInfo.CurrentMana -= Const.WoodSpellMana;
            }
        }

        public void SummonFamiliar(Vector2 loc, int type)
        {
            if (familiars.Count == 0)
            {
                FamiliarInfo familiarInfo = new FamiliarInfo();
                familiarInfo.Loc = loc;
                Familiar familiar = (Familiar)familiarInfo.Create(CursedGame);
                familiar.Master = this;
                familiars.Add(familiar);
                CursedGame.AddGob(familiar);
            }
        }
        public void DismissFamiliar(int type)
        {
            foreach (Familiar familiar in familiars)
            {
                CursedGame.RemoveGob(familiar);
            }
            familiars.Clear();
        }

        public override void control(ControlInfo cInfo)
        {
            base.control(cInfo);
            if (cInfo is MouseTrailInfo && !CursedGame.Paused)
            {
                MouseTrailInfo mInfo = (MouseTrailInfo)cInfo;
                wand.MouseType = mInfo.Type;
                wand.Strong = mInfo.Modifier;
                if (mInfo.Trail.Count > 0)
                {
                    Vector2 last = mInfo.Trail[mInfo.Trail.Count - 1];
                    if (mInfo.Type == MouseType.Offensive && mInfo.Modifier == false)
                        castLife(last);
                    if (mInfo.Type == MouseType.Defensive && mInfo.Modifier == false)
                        castWood(last);
                }
            }
            if (cInfo is GestureInfo)
            {
                GestureInfo gInfo = (GestureInfo)cInfo;
                if (gInfo.Gesture.Matches(familiarGesture))
                {
                    if (gInfo.Type == MouseType.Offensive)
                        SummonFamiliar(GobInfo.Loc + CursedGame.RandomVector2, 0);
                    if (gInfo.Type == MouseType.Defensive)
                        DismissFamiliar(0);
                }
            }
            if (cInfo is MouseCursorInfo && !CursedGame.Paused)
            {
                MouseCursorInfo mInfo = (MouseCursorInfo)cInfo;
                wand.GobInfo.Loc = mInfo.Mouse;
                wand.MouseType = mInfo.Type;
                wand.Strong = mInfo.Modifier;

                if (mInfo.Clicked && mInfo.Type == MouseType.Offensive && mInfo.Modifier == false)
                {
                    castLife(mInfo.Mouse);
                }

                if (mInfo.Clicked && mInfo.Type == MouseType.Defensive && mInfo.Modifier == false)
                {
                    castWood(mInfo.Mouse);
                }
            }
        }
        public override void takeDamage(float damage)
        {
            #region temporary
            CursedGame.FogColor = new Microsoft.Xna.Framework.Graphics.Color((byte)255, CursedGame.FogColor.G, CursedGame.FogColor.B);
            #endregion
            base.takeDamage(damage);
        }

        public override void die()
        {
            base.die();
        }

        public override void Unload()
        {
            if (keyController != null)
                CursedGame.RemoveController(keyController);
            if (mouseController != null)
                CursedGame.RemoveController(mouseController);
            if (mouseTrailController != null)
                CursedGame.RemoveController(mouseTrailController);
            if (gestureController != null)
                CursedGame.RemoveController(gestureController);
            CursedGame.RemoveGob(wand);
            base.Unload();
            CursedGame.LoadLevel("StartScreen");
        }
    }
}
