﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using System.Windows.Media;
using Cursed.view;
using Cursed.geom;
using System.Runtime.Serialization;
using Cursed.view.renders;

namespace Cursed.gob.tile
{
    public class TileInfo : GobInfo
    {
        int x, y;

        public TileInfo(int x, int y)
            : base()
        {
            this.x = x;
            this.y = y;
            Loc = new Vector2(x, y);
        }

        public int X
        {
            get { return x; }
            set
            {
                x = value;
                Loc = new Vector2(x, y);
            }
        }
        public int Y
        {
            get { return y; }
            set
            {
                y = value;
                Loc = new Vector2(x, y);
            }
        }
    }

    public class Tile : Gob
    {
        protected RenderInfo renderInfo;

        public Tile(CursedGame game, TileInfo info)
            : base(game, info)
        {
            Inert = true;
            GobInfo.Loc = new Vector2(info.X, info.Y);
        }

        public override void Load()
        {
            shape = new Rect(Vector2.Zero, 1, 1);
            renderInfo = new SpriteRenderInfo(new Bounds(GobInfo.DrawCenter, GobInfo.DrawWidth, GobInfo.DrawHeight));
            if (GobInfo.Texture != "")
                renderInfo.Texture = CursedGame.textureLib.GetTexture(GobInfo.Texture);
            else
                renderInfo.Texture = CursedGame.textureLib.GetTexture("square");

            renderInfo.Mask = info.Mask;
            addRender(CursedGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender), renderInfo);


        }

        public override void Update(GameTime gameTime)
        {
            updateTransform();
            updateRenderTransform(renderInfo);
            base.Update(gameTime);
        }

        protected void updateTransform()
        {
            if (shape != null)
                shape.setTransform(GobInfo.Loc, 0, 1);
        }

        protected void updateRenderTransform(RenderInfo rInfo)
        {
            rInfo.Transform = Microsoft.Xna.Framework.Matrix.CreateTranslation(GobInfo.Loc3);
        }

        public int X
        {
            get { return TileInfo.X; }
        }

        public int Y
        {
            get { return TileInfo.Y; }
        }

        public TileInfo TileInfo
        {
            get { return (TileInfo)info; }
        }
    }
}
