﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.gob.mob;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Media;

namespace Cursed.gob.tile
{
    public class EntranceInfo : TileInfo
    {
        string music = "improv2";

        public EntranceInfo(int x, int y)
            : base(x, y)
        {
            Depth = Const.EntranceDepth;
        }
        public override Gob Create(CursedGame game)
        {
            return new Entrance(game, this);
        }

        public string Music
        {
            get { return music; }
            set { music = value; }
        }
    }

    public class Entrance : Tile
    {
        public Entrance(CursedGame game, EntranceInfo info)
            : base(game, info)
        {
            Transcendent = true;
            Transient = false;
            Inert = true;
        }

        public override void Load()
        {
            base.Load();
            CursedGame.Music = EntranceInfo.Music;
        }

        public EntranceInfo EntranceInfo
        {
            get { return (EntranceInfo)info; }
        }
    }

    public class ExitInfo : TileInfo
    {
        string level;

        public ExitInfo(int x, int y)
            : base(x, y)
        {
            level = "";
            Depth = Const.ExitDepth;
        }

        public override Gob Create(CursedGame game)
        {
            return new Exit(game, this);
        }

        public string Level
        {
            get { return level; }
            set { level = value; }
        }
    }

    public class Exit : Tile
    {
        public Exit(CursedGame game, ExitInfo info)
            : base(game, info)
        {
            Transcendent = true;
            Transient = false;
            Inert = false;
        }

        public override void Load()
        {
            base.Load();
            renderInfo.Mask = Color.Blue;
        }

        protected void doWarp()
        {
            if (GobInfo.Label == "")
            {
                CursedGame.RemoveGob(CursedGame.DruidPlayer);
            }
            CursedGame.LoadLevel(ExitInfo.Level);
            CursedGame.SpawnLabel = ExitInfo.Label;
        }

        public override void Update(Microsoft.Xna.Framework.GameTime gameTime)
        {
            if (TouchedGobs.Contains(CursedGame.DruidPlayer))
                doWarp();
            base.Update(gameTime);
        }

        public ExitInfo ExitInfo
        {
            get { return (ExitInfo)info; }
        }
    }
}
