﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework;

namespace Cursed.input
{
    public class TextInputController : Controller
    {
        static bool canBeActive;
        bool enterWasDown;
        TextInputInfo messageInfo;
        private int cursor;
        Keys[] pressed;
        bool shiftOn;
        float shiftLag;
        public TextInputController()
        {
            cursor = -1;
            canBeActive = false;
            messageInfo = new TextInputInfo();
            messageInfo.TextInput = null;
            messageInfo.Lines = new List<string>();
            messageInfo.InProgress = null;
            messageInfo.Active = false;
            messageInfo.LineLimit = 15;
            shiftOn = false;
        }
        public bool CanBeActive
        {
            get { return canBeActive; }
            set { canBeActive = value; }
        }
        public bool Active
        {
            get { return messageInfo.Active; }
        }
        public void addLine(string line)
        {
            while (messageInfo.Lines.Count >= messageInfo.LineLimit-1)
            {
                messageInfo.Lines.RemoveAt(0);
            }
            messageInfo.Lines.Add(line);
        }
        public override void act(Microsoft.Xna.Framework.GameTime time)
        {
            if (!CanBeActive)
            {
                if (messageInfo.Active)
                    messageInfo.Active = false;
                return;
            }
            messageInfo.TextInput = null;
            bool enterIsDown = Keyboard.GetState()[Keys.Enter] == KeyState.Down;
            if (Keyboard.GetState()[Keys.LeftShift] == KeyState.Down)
            {
                shiftLag = .1f;
                shiftOn = true;
            }
            else if(shiftOn)
            {
                shiftLag -= time.ElapsedRealTime.Milliseconds * .001f;
                
                if (shiftLag <= 0)
                {
                    
                    shiftOn = false;
                }
            }
            if (!messageInfo.Active && enterWasDown && !enterIsDown)
            {
                messageInfo.Active = true;
                
            }
            else if (messageInfo.Active && enterWasDown && !enterIsDown)
            {
                messageInfo.Active = false;
                if (messageInfo.InProgress != null && messageInfo.InProgress.Length > 0)
                {
                    addLine(messageInfo.InProgress);
                    messageInfo.TextInput = messageInfo.InProgress;
                    
                    messageInfo.InProgress = "";
                  
                }
                cursor = -1;
            }
            if (messageInfo.Active)
            {
                Keys[] nowPressed= Keyboard.GetState().GetPressedKeys();
                if (pressed != null) 
                {
                    foreach(Keys key in pressed)
                    {
                        if (!nowPressed.Contains(key))
                        {
                            char k;
                            if (key == Keys.OemPeriod) k = '.';
                            else if (key == Keys.OemPlus) k = '=';
                            else
                            {
                                k = (char)key;
                                if (!shiftOn)
                                    k = Char.ToLower(k);
                            }
                            if (k == '\b')
                            {
                                if (messageInfo.InProgress.Length > 0)
                                    messageInfo.InProgress = messageInfo.InProgress.Substring(0, messageInfo.InProgress.Length - 1);
                            }
                            else if (key == Keys.Up)
                            {
                                upLine();
                            }
                            else if (key == Keys.Down)
                            {
                                downLine();
                            }
                            else if (k >= (char)32 && k <= (char)126)
                            {
                                messageInfo.InProgress += k;
                                cursor = -1;
                            }
                        }
                    }
            }
                pressed = nowPressed;
            }
            if (Target != null)
            {
                Target.control(messageInfo);
            }
            

            enterWasDown = enterIsDown;
        }
        void upLine()
        {
            if (cursor == -1 && messageInfo.Lines.Count > 0)
            {
                cursor = messageInfo.Lines.Count - 1;
                messageInfo.InProgress = messageInfo.Lines[cursor];
            }
            else if (cursor > 0)
            {
                cursor -= 1;
                messageInfo.InProgress = messageInfo.Lines[cursor];
            }
        }
        void downLine()
        {
            if (cursor == -1) return;
            if (cursor < messageInfo.Lines.Count - 1)
            {
                cursor += 1;
                messageInfo.InProgress = messageInfo.Lines[cursor];
            }
            else
            {
                messageInfo.InProgress = "";
                cursor = -1;
            }
        }
        public override void cleanup()
        {

        }
        public int LineLimit
        {
            get { return messageInfo.LineLimit; }
        }
    }
}
