﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.gob;
using Cursed.gob.mob.kob;
using Cursed.gob.mob;
using Microsoft.Xna.Framework;

namespace Cursed.input.ai
{
    public class GreedyFamiliarControl : Controller
    {
        Familiar familiar;

        public GreedyFamiliarControl(Familiar familiar)
        {
            this.familiar = familiar;
            Push(familiar);
        }

        public override void act(Microsoft.Xna.Framework.GameTime time)
        {
            float bestScore = 0;
            Gob bestGob = null;
            int attackableCount = 0;
            foreach (Gob seenGob in familiar.SeenMobs)
            {
                if (seenGob == familiar.Master)
                    continue;
                Vector2 distance = familiar.GobInfo.Loc - seenGob.GobInfo.Loc;
                float score = (float)Math.Pow(distance.Length(), -.5f);
                if (seenGob is LifeBlob)
                {
                    LifeBlob seenBlob = (LifeBlob)seenGob;
                    if (familiar.TouchedGobs.Contains(seenBlob))
                        continue;
                    score += seenBlob.LifeInfo.Amount;
                    if (seenBlob.LifeInfo.Amount < 0)
                        score = 0;
                }
                else if (seenGob is Kob)
                {
                    Kob seenKob = (Kob)seenGob;
                    if (seenKob.KobInfo.Energy <= Const.FamiliarEnergyVisibilityThreshold)
                        continue;
                    score += (seenKob.KobInfo.Energy)+1;
                    if (score > 0 && familiar.TouchedGobs.Contains(seenKob))
                    {
                        attackableCount++;
                    }
                }
                else
                {
                    score = 0;
                }


                if (score > bestScore)
                {
                    bestScore = score;
                    bestGob = seenGob;
                }
            }


            Vector2 direction = Vector2.Zero;

            if (bestGob != null)
            {
                direction = bestGob.GobInfo.Loc - familiar.GobInfo.Loc;
                direction.Normalize();
            }
            if (!(bestGob is Kob && familiar.TouchedGobs.Contains(bestGob)))
            {
                attackableCount = 0;
            }

            float bufferToBest = 0;
            if (bestGob != null)
                bufferToBest = (bestGob.GobInfo.Loc - familiar.GobInfo.Loc).Length();
            if (bestGob is Mob)
            {
                bufferToBest -= ((Mob)bestGob).MobInfo.Scale * .5f;
            }
            else
            {
                bufferToBest -= .5f;
            }
            bufferToBest -= familiar.MobInfo.Scale * .5f;


            if (Target != null)
            {
                DirectionControlInfo directionInfo = new DirectionControlInfo();
                directionInfo.Direction = Vector2.Zero;

                bool touchesMaster = familiar.Master != null && familiar.Master.TouchedGobs.Contains(bestGob);

                if (!touchesMaster && bestGob is Kob && (bestGob.GobInfo.Loc - familiar.GobInfo.Loc).Length() > familiar.GobInfo.ViewRange - 1)
                {
                    Target.control(new LungeInfo());
                }
                if (touchesMaster && bufferToBest < 1)
                {
                    directionInfo.Direction = Vector2.Transform(direction, Matrix.CreateRotationZ(MathHelper.PiOver4 + .2f));
                }
                else if (touchesMaster && bufferToBest < 2)
                {
                    directionInfo.Direction = Vector2.Transform(direction, Matrix.CreateRotationZ(MathHelper.PiOver4));
                }
                else
                {
                    directionInfo.Direction = Vector2.Transform(direction, Matrix.CreateRotationZ(.2f));
                }

                Target.control(directionInfo);
            }

            if (attackableCount > 0)
            {
                BasicAttackInfo attackInfo = new BasicAttackInfo();
                attackInfo.Direction = direction;
                if (familiar.DemonInfo.SpinAttack && attackableCount > 1)
                    attackInfo.Type = BasicAttackInfo.ATTACKTYPE.SPIN;
                if (Target != null)
                {
                    Target.control(attackInfo);
                }
            }
        }

        public override void cleanup()
        {

        }
    }
}
