﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Cursed.gob.mob.kob;
using Microsoft.Xna.Framework;

namespace Cursed.view
{
    public class HUD
    {
        List<HUDComponent> hudComponents;
        //SpriteBatch spriteBatch = null;

        public HUD()
        {
            hudComponents = new List<HUDComponent>();
        }

        public void AddComponenet(HUDComponent component)
        {
            hudComponents.Add(component);
        }

        public void Draw(SpriteBatch spriteBatch, GameTime gameTime)
        {
            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Immediate, SaveStateMode.SaveState);
            foreach (HUDComponent component in hudComponents)
            {
                component.Draw(spriteBatch, gameTime);
            }
            spriteBatch.End();
        }

    }
    public abstract class HUDComponent
    {
        Rectangle bounds;

        public Rectangle Bounds
        {
            get { return bounds; }
            set { bounds = value; }
        }

        public HUDComponent(Rectangle bounds)
        {
            this.bounds = bounds;
        }
        public abstract void Draw(SpriteBatch spriteBatch, GameTime gameTime);
    }

    public class HealthBar : HUDComponent
    {
        Kob kob;

        public HealthBar(Rectangle bounds, Kob kob)
            : base(bounds)
        {
            this.kob = kob;
        }

        public override void Draw(SpriteBatch spriteBatch, GameTime gameTime)
        {
            spriteBatch.Draw(
                CursedGame.textureLib.GetTexture("square"),
                Bounds,
                new Color(.1f, .1f, .1f));
            spriteBatch.Draw(
                CursedGame.textureLib.GetTexture("square"),
                new Rectangle(
                    Bounds.X,
                    Bounds.Y,
                    (int)((float)Bounds.Width * kob.KobInfo.CurrentHealth / kob.KobInfo.MaxHealth),
                    Bounds.Height),
                Color.Red);
        }
    }

    public class ManaBar : HUDComponent
    {
        Druid druid;

        public ManaBar(Rectangle bounds, Druid druid)
            : base(bounds)
        {
            this.druid = druid;
        }

        public override void Draw(SpriteBatch spriteBatch, GameTime gameTime)
        {
            spriteBatch.Draw(
                CursedGame.textureLib.GetTexture("square"),
                Bounds,
                new Color(.1f, .1f, .1f));
            spriteBatch.Draw(
                CursedGame.textureLib.GetTexture("square"),
                new Rectangle(
                    Bounds.X,
                    Bounds.Y,
                    (int)((float)Bounds.Width * druid.DruidInfo.CurrentMana / druid.DruidInfo.MaxMana),
                    Bounds.Height),
                Color.Blue);
        }
    }

}
