﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Cursed.gob;
using Cursed.gob.mob;
using System.Windows;
using Cursed.geom;

namespace Cursed.view
{
    public class OrthoCamera : ICamera
    {
        GameWindow window;
        const float maxDepth = -50;
        const float minDepth = 50;
        Vector3 position;
        Matrix projection;
        Matrix view;
        float screenSize;
        float zoom;
        float aspect;
        Stack<IFocus> focusStack;

        public OrthoCamera(float screenSize, GameWindow window)
        {
            this.screenSize = screenSize;
            this.window = window;
            this.zoom = 1;
            focusStack = new Stack<IFocus>();
            updateView(window);
        }

        #region focus
        public void PushFocus(IFocus focus)
        {
            focusStack.Push(focus);
        }

        public bool PopFocus(IFocus focus)
        {
            if (Focus == focus)
            {
                focusStack.Pop();
                return true;
            }
            return false;
        }

        public IFocus Focus
        {
            get {
                if (focusStack.Count == 0)
                    return null;
                return focusStack.Peek(); 
            }
        }
        #endregion

        #region accessors
        public float Zoom
        {
            get { return zoom; }
            set { zoom = value; }
        }

        public Vector2 Position {
            get { 
                return new Vector2(position.X, position.Y);
            }
            set
            {
                position.X = value.X;
                position.Y = value.Y;
                position.Z = maxDepth;
                updateView(window);
            }
        }

        public Vector3 Position3
        {
            get { return position; }
            set { position = value; }
        }

        public float ScreenSize
        {
            get { return screenSize; }        
        }

        public float Aspect
        {
            get { return aspect; }
        }

        public float ViewWidth
        {
            get
            {
                if (aspect < 1)
                {
                    return ViewSize;
                }
                return ViewSize * aspect;
            }
        }

        public float ViewHeight
        {
            get
            {
                if (aspect < 1)
                {
                    return ViewSize / aspect;
                }
                return ViewSize;
            }
        }

        public float ViewSize
        {
            get { return screenSize / Zoom; }
        }
        #endregion

        public void updateView(GameWindow window) {
            view = Matrix.CreateLookAt(new Vector3(position.X, position.Y, 0), position, Vector3.Up);
            aspect = (float)window.ClientBounds.Width / (float)window.ClientBounds.Height;

            projection = Matrix.CreateOrthographic(ViewWidth, ViewHeight, minDepth, maxDepth);           
        }

        public void trackFocus()
        {
            Vector2 dest = Vector2.Zero;
            if (Focus != null)
            {
                dest = Focus.Loc;
            }
            Vector2 cur = Position;
            Position = cur * .9f + dest * .1f;
        }

        public Matrix View
        {
            get { return view; }
        }

        public Matrix Projection
        {
            get { return projection; }
        }

        #region ICamera Members


        public Bounds Bounds
        {
            get {
                return new Bounds(Position, ViewWidth, ViewHeight);
            }
        }

        #endregion
    }
}
