﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using System.Windows;
using Cursed.geom;

namespace Cursed.view.renders
{
    public class SpriteRender : Render
    {
        private VertexDeclaration vd;

        public VertexDeclaration Vd{
            get { return vd; }
        }


        public SpriteRender(GraphicsDevice gd, Effect effect) 
            : base(effect, gd)
        {

            vd = new VertexDeclaration(gd, VertexPositionColorTexture.VertexElements);
        }


        public override void Draw(GameTime time, RenderInfo info)
        {
            ((SpriteRenderInfo)info).SetVertices();
            effect.Parameters["ShaderIndex"].SetValue(3);
            foreach (EffectPass pass in effect.CurrentTechnique.Passes)
            {
                pass.Begin();
                base.gd.VertexDeclaration = vd;
                base.gd.DrawUserPrimitives(PrimitiveType.TriangleStrip, ((SpriteRenderInfo) info).Vertices, 0, 2);
                pass.End();
            }
        }
    }

    public class SpriteRenderInfo : RenderInfo, ICloneable
    {
        private VertexPositionColorTexture[] vertices;
        Bounds bounds;

        public VertexPositionColorTexture[] Vertices
        {
            get { return vertices; }
        }
        
        public SpriteRenderInfo(Bounds bounds)
        {
            this.bounds = bounds;

            SetVertices();
        }

        public Bounds Bounds
        {
            get { return bounds; }
            set
            {
                bounds = value;
                SetVertices();
            }
        }

        public void SetVertices()
        {
            vertices = new VertexPositionColorTexture[4];
            vertices[0].Position = new Vector3(bounds.MinX, bounds.MaxY, 0);
            vertices[1].Position = new Vector3(bounds.MinX, bounds.MinY, 0);
            vertices[2].Position = new Vector3(bounds.MaxX, bounds.MaxY, 0);
            vertices[3].Position = new Vector3(bounds.MaxX, bounds.MinY, 0);

            vertices[0].TextureCoordinate = new Vector2(0, 0);
            vertices[1].TextureCoordinate = new Vector2(0, 1);
            vertices[2].TextureCoordinate = new Vector2(1, 0);
            vertices[3].TextureCoordinate = new Vector2(1, 1);

            vertices[0].Color = Mask;
            vertices[1].Color = Mask;
            vertices[2].Color = Mask;
            vertices[3].Color = Mask;
        }

        #region ICloneable Members

        public object Clone()
        {
            SpriteRenderInfo clone = new SpriteRenderInfo(this.bounds);
            clone.Mask = this.Mask;
            clone.Texture = this.Texture;
            clone.Transform = this.Transform;

            return clone;
        }

        #endregion
    }
}
